from .child_base_data import TableData, LabelData, DocData, ImageData, AudioData, VideoData, GraphData
from .data import Data
from .download import Download
from .query import QueryDigit
from .info import Information
from .core import Core


import os

# 根目录
ROOT_PATH = os.path.expanduser("~")
# ROOT_PATH = "D:\\"
# 缓存文件夹
CACHE_PATH = os.path.join(ROOT_PATH, "digit")

# Digit Web接口
## test
# INTERFACE_PATH = 'http://127.0.0.1:8000'
## product
INTERFACE_PATH = 'http://124.70.200.79:10101'


DATA_STORAGE_TYPE_DEFAULT = 11
DATA_STORAGE_TYPE = {
    # Git仓库
    11: "Gitee",
    12: "Github",
    13: "其他托管",

    # 网络云盘
    21: "百度网盘",
    22: "阿里云盘",
    23: "坚果云网盘",
    24: "谷歌网盘",
    25: "其他网盘",

    # 实验室服务器
    31: "数据服务器",
    32: "GPU服务器",
    33: "华为服务器",
    34: "其他服务器"
}

DATA_DATA_TYPE_DEFAULT = 2
DATA_DATA_TYPE = {
    0: "Only Dataset",
    1: "Only Code",
    2: "Dataset & Code"
}

DATA_LANGUAGE_DEFAULT = 0
DATA_LANGUAGE = {
    0: 'English',
    1: 'Chinese',
    2: 'English & Chinese',
    3: 'Other'
}

DATA_TASK_TYPE_DEFAULT = 31
DATA_TASK_TYPE = {
    # "Multimodal"
    11: 'Feature Extraction',
    12: 'Text-to-Image',
    13: 'Image-to-Text',
    14: 'Text-to-Video',
    15: 'Visual Question Answering',
    16: 'Document Question Answering',
    17: 'Graph Machine Learning',

    # "Computer Vision"
    21: 'Depth Estimation',
    22: 'Image Classification',
    23: 'Object Detection',
    24: 'Image Segmentation',
    25: 'Image-to-Image',
    26: 'Unconditional Image Generation',
    27: 'Video Classification',
    28: 'Zero-Shot Image Classification',

    # "Natural Language Processing"
    31: 'Text Classification',
    32: 'Token Classification',
    33: 'Table Question Answering',
    34: 'Question Answering',
    35: 'Zero-Shot Classification',
    36: 'Translation',
    37: 'Summarization',
    38: 'Conversational',
    39: 'Text Generation',
    310: 'Text2Text Generation',
    311: 'Fill-Mask',
    312: 'Sentence Similarity',

    # "Audio"
    41: 'Text-to-Speech',
    42: 'Automatic Speech Recognition',
    43: 'Audio-to-Audio',
    44: 'Audio Classification',

    # "Tabular"
    45: 'Voice Activity Detection',
    51: 'Tabular Classification',
    52: 'Tabular Regression',
    61: 'Reinforcement Learning',
    62: 'Robotics',
    # other
    0: 'other'
}



# -*- coding:utf-8 -*-
"""
core中继承了所有类的中的所有功能
"""
from .query import QueryDigit
from .info import Information
from .setting import *
from .download import Download
from .data import Data
from .child_base_data import *


class Core():

    def __init__(self):
        self.query = QueryDigit()
        self.info = Information()
        self.download = Download()
        self.data = Data()
        self.data_id = ''

    def update_api_token(self, new_token: str):
        """
        new_token: api-token字符串
        该方法来自Information类
        """
        self.info.update_api_token(new_token)

    def get_resources(self, api_type: str = 'data', id=None, detail: bool = False):
        """
        该函数用于查询digit平台现有资源，返回资源元数据
        :param api_type: 使用何种类型的api,api_type 可选{'data','card','user','account','websetting','dataid'}
        :param id:  查询特定资源时用用该资源的id进行查看
        :param detail: 是否输出可获取的所有元数据字段
        :return: queryset 查询结果
        """
        self.query.get_resources(api_type="data", id=None, detail=False)

    def get_category(self):
        """
        该函数用于查看digit平台分类体系编码与对应的实际含义，['storage_type', 'data_type', 'language', 'task_type', ]
        :return:
        """
        self.query.get_category()

    def download_repo(self, data_id_or_name: str, update: bool = False):
        """
        该函数可以根据data_id或name下载对应的资源至本地
        :param data_id_or_name:
        :param update: 是否更新缓存
        :return:
        """
        self.data_id = self.download.download_repo(data_id_or_name, update)

    def load(self, data_id_or_name: str, imp_class="DigitData"):
        """
        该函数用于加载下载的脚本代码中的DigitData类
        :return:

        :param data_id_or_name:
        :param imp_class: 导入类的类型名称
        :return: imp_class_instance 导入类的实例
        """
        data_id = ''
        if self.data_id:
            data_id = self.data_id
        else:
            data_id = data_id_or_name
        return self.data.load(data_id_or_name=data_id, imp_class=imp_class)

    def run(self):
        self.data.run()

    def all_in_one(self, data_id_or_name: str, update: bool = False, imp_class: str = "DigitData"):
        # 下载并加载repo， 相当于download_repo()和 load两个函数
        self.download_repo(data_id_or_name, update)
        self.load(data_id_or_name=self.data_id, imp_class=imp_class)
        self.run()

    def get_data_id(self):
        """
        获取当前下载仓库的 data_id
        :return: data_id:str
        """
        print(self.data_id)
        return self.data_id

    def upload(self,md_instruction_path,config_path):
        # 上传data资源
        self.data.upload(md_instruction_path,config_path)

    def delete(self, data_id_or_name):
        # 删除自己的data
        self.data.delete(data_id_or_name)







import argparse
from .core import Core

def main():
    parser = argparse.ArgumentParser(prog="digit")
    subparsers = parser.add_subparsers(dest='command')

    parser.add_argument("-u", "--update-api-token", help="设置/更新个人的API-token")
    parser.add_argument("-c", "--category", help="查看分类体系中的编码与含义", default=None)
    parser.add_argument("-d","--delete", help="根据 data_id or name 删除个人资源")


    query_parser = subparsers.add_parser('query', help='子命令：查看平台现有资源')
    query_parser.add_argument("-t", "--type", help="指定资源类型查看，可选:data/card/useraccount/websetting/dataid", default='data')
    query_parser.add_argument("-i", "--id", help="选择特定id的资源查看", default=None)
    query_parser.add_argument("-d", "--detail", help="是否查看详情，默认为否", default=False)

    download_parser = subparsers.add_parser('download', help='子命令：下载资源到本地')
    download_parser.add_argument("-i", "--id-or-name", help="根据data_id_or_name下载对应的资源至本地，默认如果已经存在则不更新资源")
    download_parser.add_argument("-u", "--update", help="更新下载的资源")

    upload_parser = subparsers.add_parser('upload',help="子命令：上传资源")
    upload_parser.add_argument("-m" "--markdown-path",help="上传的markdown文件路径")
    upload_parser.add_argument("-c","--config-path",help="上传的config.json文件路径")

    run_parser = subparsers.add_parser('run', help='子命令：下载 + 加载 + 运行资源')
    run_parser.add_argument("-i", "--id-or-name", help="根据data_id_or_name下载对应的资源至本地，默认如果已经存在则不更新资源")
    run_parser.add_argument("-u", "--update", help="是否更新下载的资源",default=False)
    run_parser.add_argument("-ip","--imp_class", help="导入类的类型名称",default="DigitData")

    args = parser.parse_args()
    core = Core()

    if args.command == "query":
        core.get_resources(api_type=args.type, id=args.id, detail=args.detail)
    elif args.command == "download":
        core.download_repo(data_id_or_name=args.id_or_name, update=args.update)
    elif args.command =="upload":
        core.upload(md_instruction_path=args.markdown_path, config_path=args.config_path)
    elif args.command == "run":
        core.all_in_one(data_id_or_name=args.id_or_name, update=args.update, imp_class=args.imp_class)
    else:
        if args.update_api_token:
            core.update_api_token(new_token=args.update_api_token)
        elif args.category:
            core.get_category()
        elif args.delete:
            core.delete(data_id_or_name=args.delete)

        else:
            print("参数错误或未输入参数：")
            parser.print_help()


if __name__ == '__main__':
    main()



import setuptools


setuptools.setup(
    name="ir_digit",
    version="1.0.0",
    author="FKLiu",
    author_email="fkliu001@outlook.com",
    description="digit package",
    long_description="digit package 与 digit web 配套使用，下载其digit web中的数据资源并在本地运行",
    long_description_content_type="text",
    url="https://pypi.org/project/ir-digit/",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires = [
        "gitpython",
        "numpy",
        "pandas",
        "matplotlib",
        "seaborn"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
            'console_scripts': [
                'digit=digit.scripts:main',
            ],
        },

)



import json

from .info import Information
from .setting import CACHE_PATH
import os
from .query import QueryDigit
import shutil
import git
import stat


class Download():
    """
    downlaod 主要就是repo下载
    初始化时检查是否存在对应的文件目录
    不存在则创建
    """

    def __init__(self):
        self.info = Information()
        self.cache_path = CACHE_PATH
        self.qd = QueryDigit()
        self._verity_cache_path()  # 初始化目录是都存在，不存在则创建，存在则无动作

    def _verity_cache_path(self):
        # 检查是否建立了该目录，没有建立则建立
        if not os.path.exists(self.cache_path):
            os.mkdir(self.cache_path)
            os.chmod(self.cache_path, stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)

    def _get_data_id(self, data_id_or_name):
        # 检测是否已经缓存, 
        qs = self.qd.get_resources(api_type="dataid", id=data_id_or_name)

        if not qs:
            print(f"data_id_or_name = {data_id_or_name}的资源不存在")
            return False
        data_id = qs[0].get('data_id')
        return data_id

    def _generate_config(self, data_obj):
        print("开始生成config.json文件")
        # {
        #     "name": "test",
        #     "url_or_path": "https://gitee.com/fkliu/digit_test.git",
        #     "storage_type": 1,
        #     "data_type": 1,
        #     "language": 1,
        #     "task_type": 11,
        #     "tags": ["示例标签1", "示例标签2"],
        #     "description": ""
        # }
        sort_json = ['data_id', 'user_id', 'url', 'name', 'author', 'storage_type', 'data_type', 'language',
                     'task_type', 'tags', 'description', 'picture', 'md_instruction', 'usage_number',
                     'permission_level', 'status', 'create_time', 'update_time']
        temp = {i: data_obj.get(i) for i in sort_json}

        with open(os.path.join(self.cache_path, data_obj.get("data_id"),'config.json'), 'w', encoding="utf-8") as f:
            json.dump(temp, f, indent=4)
        print(f"资源信息已保存至config.json：{os.path.join(self.cache_path, data_obj.get('data_id'), 'config.json')}")
        return True

    def _delete_cache(self, data_id):
        if not os.path.exists(os.path.join(self.cache_path, data_id)):
            print(f"资源{data_id} 在缓存目录中不存在")
            return False  # 不存在，未进行删除
        shutil.rmtree(os.path.join(self.cache_path, data_id))
        self.info.delete_data(data_id=data_id)  # 在缓存表中删除

        print(f"资源{data_id} 已从缓存目录中删除")
        return True

    def _update_cache(self, data_id):
        data_obj = self.qd.get_resources(api_type="data", id=data_id)[0]

        url = data_obj.get("url") if data_obj.get("url").endswith(".git") else data_obj.get("url") + ".git"  # git仓库

        print("正在下载/更新 Git远程仓库")

        if not os.path.exists(os.path.join(self.cache_path, data_id)):
            # 获取资源
            os.mkdir(os.path.join(self.cache_path, data_id))  # 创建缓存目录，以data_id命名

            try:
                git.Repo.clone_from(url, os.path.join(self.cache_path, data_id))
            except Exception as e:
                print("Git拉取远程仓库出错, 请检查url", e)


        else:
            ### 如果已存在，则进行更新操作
            try:
                git.Repo(os.path.join(self.cache_path, data_id)).remotes.origin.pull()
            except Exception as e:
                print("Git拉取远程仓库更新出错, 请检查url", e)
        print("更新完成")
        # 下载或更新完成后
        self.info.add_data(data_id=data_id)  # 在缓存表中更新
        print("更新信息缓存表信息")

        self._generate_config(data_obj=data_obj)  # 生成config.json文件

        return data_id

    def download_repo(self, data_id_or_name: str, update=False):
        # 检测是否已经缓存
        data_id = self._get_data_id(data_id_or_name=data_id_or_name)
        if update:
            self._update_cache(data_id=data_id)
        else:
            if not self.info.query_data(data_id=data_id):  # 虽然是更新，但缓存中不存在，则仍需要更新
                self._update_cache(data_id=data_id)
            else:
                print(f"资源 {data_id_or_name} 存在缓存资源中，可直接使用，如需更新资源请将调整参数update=True")
        return data_id

    def clean_cache(self):
        shutil.rmtree(self.cache_path)
        self.info.delete_all_data()
        print("已清除全部缓存")
        return True

    def clean_cache_by_id(self, data_id):
        self._delete_cache(data_id=data_id)
        print(f"资源{data_id}缓存已清除")
        return True
